
COLUMNS_MENU2205_BASIS = ['일자', '자산', '종목명', '종목']

# COLUMNS_MENU2205_PRIMARY = 

COLUMNS_MENU2205_CATEGORY_HOLDINGS_USD = ['외화 보유정보: 외화취득액', '외화 보유정보: 외화평가액', '외화 보유정보: 손익률', '외화 보유정보: 취득단가']

COLUMNS_MENU2205_CATEGORY_WEIGHT = ['비중: 주식/채권', '비중: 자산대비', '비중: 시장대비', '비중: 시장비중']

COLUMNS_MENU2205_CATEGORY_LENDING = ['대여정보: 대용/담보수량', '대여정보: 대여수량', '대여정보: 대용비율', '대여정보: 대여비율']

COLUMNS_MENU2205_CATEGORY_EVALUATION = ['평가정보: 종가',
 '평가정보: 증감',
 '평가정보: 시가수익률',
 '평가정보: 증감',
 '평가정보: 3사평균',
 '평가정보: 4사평균',
 '평가정보: KBP가격',
 '평가정보: NICE가격',
 '평가정보: KIS가격',
 '평가정보: FNP가격',
 '평가정보: 3사평균YTM',
 '평가정보: 4사평균YTM',
 '평가정보: KBP YTM',
 '평가정보: NICE YTM',
 '평가정보: KIS YTM',
 '평가정보: FNP YTM']

COLUMNS_MENU2205_CATEGORY_INFO_STOCK = ['종목정보: 표면금리',
 '종목정보: 할인율',
 '종목정보: 매입수익률',
 '종목정보: 회계적수익률',
 '종목정보: 발행일',
 '종목정보: 만기일',
 '종목정보: 편입일',
 '종목정보: 업종구분/보증기관',
 '종목정보: 발행국가',
 '종목정보: 보증기관',
 '종목정보: 베타',
 '종목정보: 지수가중베타',
 '종목정보: 듀레이션',
 '종목정보: 컨벡시티',
 '종목정보: KBP듀레이션',
 '종목정보: KBP컨벡시티',
 '종목정보: KIS듀레이션',
 '종목정보: KIS컨벡시티',
 '종목정보: NICE듀레이션',
 '종목정보: NICE컨벡시티',
 '종목정보: FNP듀레이션',
 '종목정보: FNP컨벡시티',
 '종목정보: 신용등급',
 '종목정보: 잔존일수',
 '종목정보: 표시통화',
 '종목정보: 옵션행사일',
 '종목정보: 발행기관',
 '종목정보: 이자지급방법',
 '종목정보: 자산분류',
 '종목정보: 시가총액']

COLUMNS_MENU2205_CATEGORY_INFO_ASSET = ['자산정보: 순자산',
 '자산정보: 총자산',
 '자산정보: 기준가격',
 '자산정보: 증감',
 '자산정보: ISIN코드',
 '자산정보: TICKER']

ASSET_CATEGORIES = ['국내주식', '국내채권', 'REPO 매수', 'REPO', '국내현금', '국내선물', '국내수익증권',
       '국내수익증권(ETF)', '외화주식', '외화현금성', '외화스왑', '기타', '기타(총계정원장)', '총계정원장',
       '차입금']


# df.loc[df['name']=='그래피2우선주(전환상환)', 'hotfix'] = '그래피'
# df.loc[df['name']=='차바이오텍3우선주', 'hotfix'] = '차바이오텍'
# df.loc[df['name']=='두나무(주)', 'hotfix'] = '두나무'



import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
logger.info("shining_pebbles package initialized")

from .collection_of_utils import *
from .dataset_cleanser import *



from .office_system_dataset_cleanser import *
from .menu2205_cleanser import *
from .menu2205_consts import *
from .menu2205_loader import *
from .menu2205_utils import *
from .alias_functions import *
from .path_director import *


from .menu2205_loader import open_df_menu2205_snapshot, open_df_menu2205
from .menu2205_cleanser import *
from .menu2205_consts import ASSET_CATEGORIES
from .menu2205_exceptions import MAPPING_NAMES_TO_CORPNAMES_FOR_EXCEPTIONS

def get_preprocessed_df_menu2205_snapshot(date_ref=None, data_source='local'):
    return preprocess_menu2205(open_df_menu2205_snapshot(date_ref=date_ref, data_source=data_source))

def get_df_menu2205(fund_code, date_ref=None, from_local=True):
    return preprocess_menu2205(open_df_menu2205(fund_code=fund_code, date_ref=date_ref, from_local=from_local))

def get_df_asset_menu2205_snapshot(date_ref=None, data_source='local'):
    menu2205 = get_preprocessed_df_menu2205_snapshot(date_ref=date_ref, data_source=data_source)
    df = {category: menu2205[menu2205['자산']==category] for category in ASSET_CATEGORIES}
    return df

def get_df_asset_menu2205(fund_code, date_ref=None):
    menu2205 = get_df_menu2205(fund_code=fund_code, date_ref=date_ref)
    df = {category: menu2205[menu2205['자산']==category] for category in ASSET_CATEGORIES}
    return df

def get_df_stock_snapshot(date_ref=None):
    df = get_df_asset_menu2205_snapshot(date_ref=date_ref)[ASSET_CATEGORIES[0]]
    df = df.reset_index(drop=True)
    return df

def get_df_stock(fund_code, date_ref=None):
    df = get_df_asset_menu2205(fund_code=fund_code, date_ref=date_ref)
    df = df.reset_index(drop=True)
    return df


def get_mapping_tickers_to_names():
    stocks = get_df_stock_snapshot().iloc[:-1]
    stocks['ticker'] = stocks['종목'].apply(lambda x: x[3:-3])
    cols_to_keep = ['ticker', '종목명']
    stocks = stocks[cols_to_keep].rename(columns={'종목명': 'name'})
    mapping_tickers_to_names = stocks.set_index('ticker').to_dict()['name']
    return mapping_tickers_to_names

def get_mapping_names_to_corpnames():
    stocks = get_df_stock_snapshot().iloc[:-1]
    cols_to_keep = ['종목명', '종목정보: 발행기관',]
    stocks = stocks[cols_to_keep].rename(columns={'종목명': 'name', '종목정보: 발행기관': 'corpname'})
    mapping_names_to_corpnames = stocks.set_index('name').to_dict()['corpname']
    return mapping_names_to_corpnames

def get_df_ticker_name_corpname(date_ref=None, exceptions=True):
    df = get_df_stock_snapshot(date_ref=date_ref).iloc[:-1]
    df['ticker'] = df['종목'].apply(lambda x: x[3:-3])
    cols_to_keep = ['ticker', '종목명', '종목정보: 발행기관']
    df = df[cols_to_keep].rename(columns={'종목명': 'name', '종목정보: 발행기관': 'corpname'})
    if exceptions:
        df = apply_exceptions_to_df_ticker_name_corpname(df)
    return df

def apply_exceptions_to_df_ticker_name_corpname(df):
    for name, corpname in MAPPING_NAMES_TO_CORPNAMES_FOR_EXCEPTIONS.items():
        df.loc[df['name']==name, 'hotfix'] = corpname
    return df




def parse_commaed_number(number_str):
    if isinstance(number_str, (int, float)):
        return number_str
    else:
        return float(number_str.replace(',', ''))
   
def force_int(number):
    if isinstance(number, float) and number.is_integer():
        return int(number)
    return number


import pandas as pd

def eliminate_unnamed_in_col(col):
    col = col if 'Unnamed' not in col else None
    return col

def combine_col_prefix_suffix(row, conjunction=None):
    conjunction = ':' if conjunction is None else conjunction
    return f"{row['col_prefix']}{conjunction} {row['col_suffix']}" if row['col_prefix'] is not None else row['col_suffix']

def get_df_col_preprocessed(df):
    cols_raw = df.columns
    cols_in_row1 = df.iloc[0].tolist()
    dct = {
        'col_raw': cols_raw,
        'col_in_row1': cols_in_row1
    }
    df_cols = pd.DataFrame(dct)
    df_cols['col_prefix'] = df_cols['col_raw'].map(eliminate_unnamed_in_col).ffill()
    df_cols['col_suffix'] = df_cols['col_in_row1']
    df_cols['col_preprocessed'] = df_cols.apply(lambda row: combine_col_prefix_suffix(row), axis=1)
    return df_cols

def preprocess_columns_in_multi_columned_df(df):
    df_cols = get_df_col_preprocessed(df)
    cols_preprocessed = [col for col in df_cols['col_preprocessed'].tolist() if col is not None]
    return cols_preprocessed

def classify_category_in_multi_columned_df(df):
    df_cols = get_df_col_preprocessed(df)
    cols_category = df_cols['col_prefix'].unique().tolist()
    return cols_category

def preprocess_multi_columned_df(df):
    df = df.copy()
    cols_preprocessed = preprocess_columns_in_multi_columned_df(df)
    df.columns = cols_preprocessed
    df = df.drop([0])
    return df


import os

def get_file_path(file_folder, file_name):
    return os.path.join(file_folder, file_name)

ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
MODULE_DIR = os.path.join(ROOT_DIR)

DATA_DIR = './data'
FILE_FOLDER_MENU2205 = 'dataset-menu2205'
FILE_FOLDER_MENU2205_SNAPSHOT = 'dataset-menu2205-snapshot'

file_folder = {
    'menu2205': get_file_path(DATA_DIR, FILE_FOLDER_MENU2205),
    'menu2205-snapshot': get_file_path(DATA_DIR, FILE_FOLDER_MENU2205_SNAPSHOT),
}



MAPPING_NAMES_TO_CORPNAMES_FOR_EXCEPTIONS = {
    '그래피2우선주(전환상환)': '그래피',
    '차바이오텍3우선주': '차바이오텍',
    '두나무(주)': '두나무',
    '윈텍글로비스보통주': '윈텍글로비스',
    '에버베스트헬리오스파워PEF(241211)': '일렉트로엠',
    '메를로랩 보통주': '메를로랩',
    '규원테크 1우선주 240708 (002)': '규원테크',
    '글라세움 4우선주 221014 (002)': '글라세움',
    '이피캠텍 주식회사 보통주': '이피캠텍',
    '비브로스 보통주': '비브로스',
}


from .path_director import file_folder
from shining_pebbles import open_df_in_file_folder_by_regex
from aws_s3_controller import open_df_in_bucket_by_regex

def open_df_menu2205_snapshot(date_ref=None, data_source='local'):
    regex = f'menu2205-code000000-at{date_ref.replace("-", "")}' if date_ref else 'menu2205-code000000-at'
    if data_source == 'local':
        df = open_df_in_file_folder_by_regex(file_folder=file_folder['menu2205-snapshot'], regex=regex).reset_index().rename(columns={'index': 'Unnamed: 0'})
    elif data_source in ['aws', 's3', 'bucket']:
        df = open_df_in_bucket_by_regex(bucket='dataset-system', bucket_prefix='dataset-menu2205-snapshot', regex=regex)
    return df

def open_df_menu2205(fund_code, date_ref=None, from_local=True):
    regex = f'menu2205-code{fund_code}-at{date_ref.replace("-", "")}' if date_ref else f'menu2205-code{fund_code}-at'
    if from_local:
        df = open_df_in_file_folder_by_regex(file_folder=file_folder['menu2205'], regex=regex)
    else:
        df = open_df_in_bucket_by_regex(bucket='dataset-system', bucket_prefix='dataset-menu2205', regex=regex)
    return df


from .office_system_dataset_cleanser import preprocess_columns_in_multi_columned_df, preprocess_multi_columned_df, classify_category_in_multi_columned_df
from .office_system_dataset_utils import parse_commaed_number, force_int

preprocess_cols_menu2205 = preprocess_columns_in_multi_columned_df
classify_columns_menu2205 = classify_category_in_multi_columned_df

format_commaed_number = parse_commaed_number
format_integer = force_int